@@ -52,6 +52,9 @@ using internal::CopyBitmap;
 
 namespace {
 
+/// \brief Clean offsets when their null_count is greater than 0
+///
+/// \pre offsets.null_count() > 0
 template <typename TYPE>
 Result<BufferVector> CleanListOffsets(const std::shared_ptr<Buffer>& validity_buffer,
                                       const Array& offsets, MemoryPool* pool) {
@@ -59,43 +62,36 @@ Result<BufferVector> CleanListOffsets(const std::shared_ptr<Buffer>& validity_bu
   using OffsetArrowType = typename CTypeTraits<offset_type>::ArrowType;
   using OffsetArrayType = typename TypeTraits<OffsetArrowType>::ArrayType;
 
-  const auto& typed_offsets = checked_cast<const OffsetArrayType&>(offsets);
+  DCHECK_GT(offsets.null_count(), 0);
   const int64_t num_offsets = offsets.length();
 
-  DCHECK(validity_buffer == nullptr || offsets.null_count() == 0)
-      << "When a validity_buffer is passed, offsets must have no nulls";
-
-  if (offsets.null_count() > 0) {
-    if (!offsets.IsValid(num_offsets - 1)) {
-      return Status::Invalid("Last list offset should be non-null");
-    }
-
-    ARROW_ASSIGN_OR_RAISE(auto clean_offsets,
-                          AllocateBuffer(num_offsets * sizeof(offset_type), pool));
-
-    // Copy valid bits, ignoring the final offset (since for a length N list array,
-    // we have N + 1 offsets)
-    ARROW_ASSIGN_OR_RAISE(
-        auto clean_validity_buffer,
-        offsets.null_bitmap()->CopySlice(0, bit_util::BytesForBits(num_offsets - 1)));
-
-    const offset_type* raw_offsets = typed_offsets.raw_values();
-    auto clean_raw_offsets =
-        reinterpret_cast<offset_type*>(clean_offsets->mutable_data());
-
-    // Must work backwards so we can tell how many values were in the last non-null value
-    offset_type current_offset = raw_offsets[num_offsets - 1];
-    for (int64_t i = num_offsets - 1; i >= 0; --i) {
-      if (offsets.IsValid(i)) {
-        current_offset = raw_offsets[i];
-      }
-      clean_raw_offsets[i] = current_offset;
-    }
-
-    return BufferVector({std::move(clean_validity_buffer), std::move(clean_offsets)});
+  if (!offsets.IsValid(num_offsets - 1)) {
+    return Status::Invalid("Last list offset should be non-null");
   }
 
-  return BufferVector({validity_buffer, typed_offsets.values()});
+  ARROW_ASSIGN_OR_RAISE(auto clean_offsets,
+                        AllocateBuffer(num_offsets * sizeof(offset_type), pool));
+
+  // Copy valid bits, ignoring the final offset (since for a length N list array,
+  // we have N + 1 offsets)
+  ARROW_ASSIGN_OR_RAISE(
+      auto clean_validity_buffer,
+      CopyBitmap(pool, offsets.null_bitmap()->data(), offsets.offset(), num_offsets - 1));
+
+  const offset_type* raw_offsets =
+      checked_cast<const OffsetArrayType&>(offsets).raw_values();
+  auto clean_raw_offsets = reinterpret_cast<offset_type*>(clean_offsets->mutable_data());
+
+  // Must work backwards so we can tell how many values were in the last non-null value
+  offset_type current_offset = raw_offsets[num_offsets - 1];
+  for (int64_t i = num_offsets - 1; i >= 0; --i) {
+    if (offsets.IsValid(i)) {
+      current_offset = raw_offsets[i];
+    }
+    clean_raw_offsets[i] = current_offset;
+  }
+
+  return BufferVector({std::move(clean_validity_buffer), std::move(clean_offsets)});
 }
 
 template <typename TYPE>
@@ -124,14 +120,21 @@ Result<std::shared_ptr<typename TypeTraits<TYPE>::ArrayType>> ListArrayFromArray
     return Status::NotImplemented("Null bitmap with offsets slice not supported.");
   }
 
-  std::shared_ptr<Buffer> offset_buf, validity_buf;
-  ARROW_ASSIGN_OR_RAISE(auto buffers, CleanListOffsets<TYPE>(null_bitmap, offsets, pool));
-  int64_t null_count_ = null_bitmap ? null_count : offsets.null_count();
+  // Clean the offsets if they contain nulls.
+  if (offsets.null_count() > 0) {
+    ARROW_ASSIGN_OR_RAISE(auto buffers,
+                          CleanListOffsets<TYPE>(null_bitmap, offsets, pool));
+    auto data = ArrayData::Make(type, offsets.length() - 1, std::move(buffers),
+                                {values.data()}, offsets.null_count(), /*offset=*/0);
+    return std::make_shared<ArrayType>(std::move(data));
+  }
 
-  std::shared_ptr<arrow::ArrayData> internal_data = ArrayData::Make(
-      type, offsets.length() - 1, std::move(buffers), null_count_, offsets.offset());
-  internal_data->child_data.push_back(values.data());
-  return std::make_shared<ArrayType>(internal_data);
+  using OffsetArrayType = typename TypeTraits<OffsetArrowType>::ArrayType;
+  const auto& typed_offsets = checked_cast<const OffsetArrayType&>(offsets);
+  auto buffers = BufferVector({std::move(null_bitmap), typed_offsets.values()});
+  auto data = ArrayData::Make(type, offsets.length() - 1, std::move(buffers),
+                              {values.data()}, null_count, offsets.offset());
+  return std::make_shared<ArrayType>(std::move(data));
 }
 
 static std::shared_ptr<Array> SliceArrayWithOffsets(const Array& array, int64_t begin,
@@ -374,10 +377,18 @@ Result<std::shared_ptr<Array>> MapArray::FromArraysInternal(
     return Status::Invalid("Map key and item arrays must be equal length");
   }
 
-  ARROW_ASSIGN_OR_RAISE(auto buffers, CleanListOffsets<MapType>(NULLPTR, *offsets, pool));
+  if (offsets->null_count() > 0) {
+    ARROW_ASSIGN_OR_RAISE(auto buffers,
+                          CleanListOffsets<MapType>(NULLPTR, *offsets, pool));
+    return std::make_shared<MapArray>(type, offsets->length() - 1, std::move(buffers),
+                                      keys, items, offsets->null_count(), 0);
+  }
 
+  using OffsetArrayType = typename TypeTraits<OffsetArrowType>::ArrayType;
+  const auto& typed_offsets = checked_cast<const OffsetArrayType&>(*offsets);
+  auto buffers = BufferVector({nullptr, typed_offsets.values()});
   return std::make_shared<MapArray>(type, offsets->length() - 1, std::move(buffers), keys,
-                                    items, offsets->null_count(), offsets->offset());
+                                    items, /*null_count=*/0, offsets->offset());
 }
 
 Result<std::shared_ptr<Array>> MapArray::FromArrays(const std::shared_ptr<Array>& offsets,
@@ -470,11 +481,11 @@ const FixedSizeListType* FixedSizeListArray::list_type() const {
   return checked_cast<const FixedSizeListType*>(data_->type.get());
 }
 
-std::shared_ptr<DataType> FixedSizeListArray::value_type() const {
+const std::shared_ptr<DataType>& FixedSizeListArray::value_type() const {
   return list_type()->value_type();
 }
 
-std::shared_ptr<Array> FixedSizeListArray::values() const { return values_; }
+const std::shared_ptr<Array>& FixedSizeListArray::values() const { return values_; }
 
 Result<std::shared_ptr<Array>> FixedSizeListArray::FromArrays(
     const std::shared_ptr<Array>& values, int32_t list_size) {
@@ -616,6 +627,22 @@ std::shared_ptr<Array> StructArray::GetFieldByName(const std::string& name) cons
   return i == -1 ? nullptr : field(i);
 }
 
+Status StructArray::CanReferenceFieldByName(const std::string& name) const {
+  if (GetFieldByName(name) == nullptr) {
+    return Status::Invalid("Field named '", name,
+                           "' not found or not unique in the struct.");
+  }
+  return Status::OK();
+}
+
+Status StructArray::CanReferenceFieldsByNames(
+    const std::vector<std::string>& names) const {
+  for (const auto& name : names) {
+    ARROW_RETURN_NOT_OK(CanReferenceFieldByName(name));
+  }
+  return Status::OK();
+}
+
 Result<ArrayVector> StructArray::Flatten(MemoryPool* pool) const {
   ArrayVector flattened;
   flattened.resize(data_->child_data.size());
