@@ -54,20 +54,34 @@ using rj::StringBuffer;
 using std::string_view;
 using Writer = rj::Writer<StringBuffer>;
 
+struct GenerateOptions {
+  // Probability of a field being written
+  double field_probability = 1.0;
+  // Probability of a value being null
+  double null_probability = 0.2;
+  // Whether to randomize the order of written fields
+  bool randomize_field_order = false;
+
+  static constexpr GenerateOptions Defaults() { return GenerateOptions{}; }
+};
+
 inline static Status OK(bool ok) { return ok ? Status::OK() : Status::Invalid(""); }
 
 template <typename Engine>
-inline static Status Generate(const std::shared_ptr<DataType>& type, Engine& e,
-                              Writer* writer);
+inline static Status Generate(
+    const std::shared_ptr<DataType>& type, Engine& e, Writer* writer,
+    const GenerateOptions& options = GenerateOptions::Defaults());
 
 template <typename Engine>
-inline static Status Generate(const std::vector<std::shared_ptr<Field>>& fields,
-                              Engine& e, Writer* writer);
+inline static Status Generate(
+    const std::vector<std::shared_ptr<Field>>& fields, Engine& e, Writer* writer,
+    const GenerateOptions& options = GenerateOptions::Defaults());
 
 template <typename Engine>
-inline static Status Generate(const std::shared_ptr<Schema>& schm, Engine& e,
-                              Writer* writer) {
-  return Generate(schm->fields(), e, writer);
+inline static Status Generate(
+    const std::shared_ptr<Schema>& schm, Engine& e, Writer* writer,
+    const GenerateOptions& options = GenerateOptions::Defaults()) {
+  return Generate(schm->fields(), e, writer, options);
 }
 
 template <typename Engine>
@@ -99,7 +113,7 @@ struct GenerateImpl {
   template <typename T>
   enable_if_base_binary<T, Status> Visit(const T&) {
     auto size = std::poisson_distribution<>{4}(e);
-    std::uniform_int_distribution<uint16_t> gen_char(32, 127);  // FIXME generate UTF8
+    std::uniform_int_distribution<uint16_t> gen_char(32, 126);  // FIXME generate UTF8
     std::string s(size, '\0');
     for (char& ch : s) ch = static_cast<char>(gen_char(e));
     return OK(writer.String(s.c_str()));
@@ -109,11 +123,13 @@ struct GenerateImpl {
   enable_if_list_like<T, Status> Visit(const T& t) {
     auto size = std::poisson_distribution<>{4}(e);
     writer.StartArray();
-    for (int i = 0; i < size; ++i) RETURN_NOT_OK(Generate(t.value_type(), e, &writer));
+    for (int i = 0; i < size; ++i) {
+      RETURN_NOT_OK(Generate(t.value_type(), e, &writer, options));
+    }
     return OK(writer.EndArray(size));
   }
 
-  Status Visit(const StructType& t) { return Generate(t.fields(), e, &writer); }
+  Status Visit(const StructType& t) { return Generate(t.fields(), e, &writer, options); }
 
   Status Visit(const DayTimeIntervalType& t) { return NotImplemented(t); }
 
@@ -129,35 +145,62 @@ struct GenerateImpl {
 
   Status Visit(const UnionType& t) { return NotImplemented(t); }
 
+  Status Visit(const RunEndEncodedType& t) { return NotImplemented(t); }
+
   Status NotImplemented(const DataType& t) {
     return Status::NotImplemented("random generation of arrays of type ", t);
   }
 
   Engine& e;
   rj::Writer<rj::StringBuffer>& writer;
+  const GenerateOptions& options;
 };
 
 template <typename Engine>
 inline static Status Generate(const std::shared_ptr<DataType>& type, Engine& e,
-                              Writer* writer) {
-  if (std::uniform_real_distribution<>{0, 1}(e) < .2) {
-    // one out of 5 chance of null, anywhere
+                              Writer* writer, const GenerateOptions& options) {
+  if (std::bernoulli_distribution(options.null_probability)(e)) {
     writer->Null();
     return Status::OK();
   }
-  GenerateImpl<Engine> visitor = {e, *writer};
+  GenerateImpl<Engine> visitor = {e, *writer, options};
   return VisitTypeInline(*type, &visitor);
 }
 
 template <typename Engine>
 inline static Status Generate(const std::vector<std::shared_ptr<Field>>& fields,
-                              Engine& e, Writer* writer) {
+                              Engine& e, Writer* writer, const GenerateOptions& options) {
   RETURN_NOT_OK(OK(writer->StartObject()));
-  for (const auto& f : fields) {
-    writer->Key(f->name().c_str());
-    RETURN_NOT_OK(Generate(f->type(), e, writer));
+
+  int num_fields = 0;
+  auto write_field = [&](const Field& f) {
+    ++num_fields;
+    writer->Key(f.name().c_str());
+    return Generate(f.type(), e, writer, options);
+  };
+
+  std::bernoulli_distribution bool_dist(options.field_probability);
+  if (options.randomize_field_order) {
+    std::vector<size_t> indices;
+    indices.reserve(static_cast<size_t>(fields.size() * options.field_probability));
+    for (size_t i = 0; i < fields.size(); ++i) {
+      if (bool_dist(e)) {
+        indices.push_back(i);
+      }
+    }
+    std::shuffle(indices.begin(), indices.end(), e);
+    for (auto i : indices) {
+      RETURN_NOT_OK(write_field(*fields[i]));
+    }
+  } else {
+    for (const auto& f : fields) {
+      if (bool_dist(e)) {
+        RETURN_NOT_OK(write_field(*f));
+      }
+    }
   }
-  return OK(writer->EndObject(static_cast<int>(fields.size())));
+
+  return OK(writer->EndObject(num_fields));
 }
 
 inline static Status MakeStream(string_view src_str,
