@@ -562,8 +562,15 @@ class TypedStatisticsImpl : public TypedStatistics<DType> {
     } else {
       this->has_null_count_ = false;
     }
-    // Clear has_distinct_count_ as distinct count cannot be merged.
-    has_distinct_count_ = false;
+    if (has_distinct_count_ && other.HasDistinctCount() &&
+        (distinct_count() == 0 || other.distinct_count() == 0)) {
+      // We can merge distinct counts if either side is zero.
+      statistics_.distinct_count =
+          std::max(statistics_.distinct_count, other.distinct_count());
+    } else {
+      // Otherwise clear has_distinct_count_ as distinct count cannot be merged.
+      this->has_distinct_count_ = false;
+    }
     // Do not clear min/max here if the other side does not provide
     // min/max which may happen when other is an empty stats or all
     // its values are null and/or NaN.
@@ -621,7 +628,9 @@ class TypedStatisticsImpl : public TypedStatistics<DType> {
       // num_values_ is reliable and it means number of non-null values.
       s.all_null_value = num_values_ == 0;
     }
-    // TODO (GH-36505): distinct count is not encoded for now.
+    if (HasDistinctCount()) {
+      s.set_distinct_count(this->distinct_count());
+    }
     return s;
   }
 
@@ -762,7 +771,7 @@ void TypedStatisticsImpl<DType>::PlainEncode(const T& src, std::string* dst) con
   encoder->Put(&src, 1);
   auto buffer = encoder->FlushValues();
   auto ptr = reinterpret_cast<const char*>(buffer->data());
-  dst->assign(ptr, buffer->size());
+  dst->assign(ptr, static_cast<size_t>(buffer->size()));
 }
 
 template <typename DType>
